@@ -17,33 +17,37 @@ KIND, either express or implied.  See the License for the
 specific language governing permissions and limitations
 under the License.
 """
-from riak import RiakError
 import base64
 import random
 import threading
 import platform
 import os
+import json
 from feature_detect import FeatureDetection
 
 
 class RiakTransport(FeatureDetection):
     """
-    Class to encapsulate transport details
+    Class to encapsulate transport details and methods. All protocol
+    transports are subclasses of this class.
     """
 
-    # Subclasses should specify their API level.
-    #   * missing or 1: the API used up and through 1.3.x.
-    #   * 2: the API introduced with 1.4.x
-    #
-    # api = 2
+    def _get_client_id(self):
+        return self._client_id
+
+    def _set_client_id(self, value):
+        self._client_id = value
+
+    client_id = property(_get_client_id, _set_client_id,
+                         doc="""the client ID for this connection""")
 
     @classmethod
     def make_random_client_id(self):
         """
         Returns a random client identifier
         """
-        return 'py_%s' % base64.b64encode(
-                str(random.randint(1, 0x40000000)))
+        return ('py_%s' %
+                base64.b64encode(str(random.randint(1, 0x40000000))))
 
     @classmethod
     def make_fixed_client_id(self):
@@ -58,68 +62,80 @@ class RiakTransport(FeatureDetection):
     def ping(self):
         """
         Ping the remote server
-        @return boolean
         """
         raise NotImplementedError
 
-    def get(self, robj, r=None, vtag=None):
+    def get(self, robj, r=None, pr=None, timeout=None):
         """
-        Serialize get request and deserialize response
-        @return (vclock=None, [(metadata, value)]=None)
+        Fetches an object.
         """
         raise NotImplementedError
 
-    def put(self, robj, w=None, dw=None, return_body=True):
+    def put(self, robj, w=None, dw=None, pw=None, return_body=None,
+            if_none_match=None, timeout=None):
         """
-        Serialize put request and deserialize response - if 'content'
-        is true, retrieve the updated metadata/content
-        @return (vclock=None, [(metadata, value)]=None)
+        Stores an object.
         """
         raise NotImplementedError
 
-    def put_new(self, robj, w=None, dw=None, return_meta=True):
-        """Put a new object into the Riak store, returning its (new) key.
-
-        If return_meta is False, then the vlock and metadata return values
-        will be None.
-
-        @return (key, vclock, metadata)
+    def delete(self, robj, rw=None, r=None, w=None, dw=None, pr=None,
+               pw=None, timeout=None):
+        """
+        Deletes an object.
         """
         raise NotImplementedError
 
-    def delete(self, robj, rw=None):
+    def get_buckets(self, timeout=None):
         """
-        Serialize delete request and deserialize response
-        @return true
+        Gets the list of buckets as strings.
         """
         raise NotImplementedError
 
-    def get_buckets(self):
+    def stream_buckets(self, timeout=None):
         """
-        Serialize get buckets request and deserialize response
-        @return dict()
+        Streams the list of buckets through an iterator
         """
         raise NotImplementedError
 
     def get_bucket_props(self, bucket):
         """
-        Serialize get bucket property request and deserialize response
-        @return dict()
+        Fetches properties for the given bucket.
         """
         raise NotImplementedError
 
     def set_bucket_props(self, bucket, props):
         """
-        Serialize set bucket property request and deserialize response
-        bucket = bucket object
-        props = dictionary of properties
-        @return boolean
+        Sets properties on the given bucket.
+        """
+        raise NotImplementedError
+
+    def clear_bucket_props(self, bucket):
+        """
+        Reset bucket properties to their defaults
+        """
+        raise NotImplementedError
+
+    def get_keys(self, bucket, timeout=None):
+        """
+        Lists all keys within the given bucket.
+        """
+        raise NotImplementedError
+
+    def stream_keys(self, bucket, timeout=None):
+        """
+        Streams the list of keys for the bucket through an iterator.
         """
         raise NotImplementedError
 
     def mapred(self, inputs, query, timeout=None):
         """
-        Serialize map/reduce request
+        Sends a MapReduce request synchronously.
+        """
+        raise NotImplementedError
+
+    def stream_mapred(self, inputs, query, timeout=None):
+        """
+        Streams the results of a MapReduce request through an iterator.
         """
         raise NotImplementedError
 
@@ -143,12 +159,46 @@ class RiakTransport(FeatureDetection):
         """
         raise NotImplementedError
 
-    def get_index(self, bucket, index, startkey, endkey=None):
+    def get_index(self, bucket, index, startkey, endkey=None,
+                  return_terms=None, max_results=None, continuation=None):
         """
         Performs a secondary index query.
         """
         raise NotImplementedError
 
+    def stream_index(self, bucket, index, startkey, endkey=None,
+                     return_terms=None, max_results=None, continuation=None):
+        """
+        Streams a secondary index query.
+        """
+        raise NotImplementedError
+
+    def fulltext_add(self, index, *docs):
+        """
+        Adds documents to the full-text index.
+        """
+        raise NotImplementedError
+
+    def fulltext_delete(self, index, docs=None, queries=None):
+        """
+        Removes documents from the full-text index.
+        """
+        raise NotImplementedError
+
+    def get_counter(self, bucket, key, r=None, pr=None, basic_quorum=None,
+                    notfound_ok=None):
+        """
+        Gets the value of a counter.
+        """
+        raise NotImplementedError
+
+    def update_counter(self, bucket, key, value, w=None, dw=None, pw=None,
+                       returnvalue=False):
+        """
+        Updates a counter by the given value.
+        """
+        raise NotImplementedError
+
     def _search_mapred_emu(self, index, query):
         """
         Emulates a search request via MapReduce. Used in the case
@@ -164,7 +214,7 @@ class RiakTransport(FeatureDetection):
         mr_result = self.mapred({'module': 'riak_search',
                                  'function': 'mapred_search',
                                  'arg': [index, query]},
-                           phases)
+                                phases)
         result = {'num_found': len(mr_result),
                   'max_score': 0.0,
                   'docs': []}
@@ -197,28 +247,16 @@ class RiakTransport(FeatureDetection):
                                   'index': index,
                                   'key': startkey},
                                  phases)
-        return [key for bucket, key in result]
+        return [key for resultbucket, key in result]
 
-    def store_file(self, key, content_type="application/octet-stream",
-                   content=None):
-        """
-        Store a large piece of data in luwak.
-        key = the key/filename for the object
-        content_type = the object's content type
-        content = the object's data
-        """
-        raise NotImplementedError
+    def _construct_mapred_json(self, inputs, query, timeout=None):
+        if not self.phaseless_mapred() and (query is None or len(query) is 0):
+            raise Exception(
+                'Phase-less MapReduce is not supported by Riak node')
 
-    def get_file(self, key):
-        """
-        Get an object from luwak.
-        key = the object's key
-        """
-        raise NotImplementedError
+        job = {'inputs': inputs, 'query': query}
+        if timeout is not None:
+            job['timeout'] = timeout
 
-    def delete_file(self, key):
-        """
-        Delete an object in luwak.
-        key = the object's key
-        """
-        raise NotImplementedError
+        content = json.dumps(job)
+        return content
